from code_data_science import (
    data_table as dt,
    unique_dictionaries as ud,
    tree_data_grid,
)


def dataframe_to_tree(df):
    df["repositoryPath"].fillna("", inplace=True)
    df["version"].fillna("", inplace=True)
    df["requestedVersion"].fillna("", inplace=True)

    tree = ud.UniqueDictionaries()

    for _, row in df.iterrows():
        parent_path = row["repositoryPath"]
        child_name = row["name"]

        tree.add(
            {
                "path": (parent_path, child_name),
                "version": f"{row['requestedVersion']}",
            }
        )

    return tree.to_list()


def create_tree_view():
    df = dt.read_csv("../samples/dependency_tree_view_nodejs.csv")
    df = df.drop_duplicates(subset=["repositoryPath", "name"])
    tree_data = dataframe_to_tree(df)

    tree_data_grid.display(
        tree_data,
        "Projects",
        [
            {"field": "version", "headerName": "Version requested", "minWidth": 200},
        ],
    )


"""Shared utilities for code quality metric visualizations."""

import pandas as pd
import plotly.graph_objects as go

# Standard color scales
HEALTH_COLORSCALE = ["#EF5350", "#FF8A65", "#FFE082", "#A5D6A7", "#4CAF50"]  # red to green
SEVERITY_COLORS = {
    "CRITICAL": "#EF5350",
    "HIGH": "#FF8A65",
    "MEDIUM": "#FFE082",
    "LOW": "#A5D6A7",
}
SEVERITY_ORDER = ["LOW", "MEDIUM", "HIGH", "CRITICAL"]


def filter_repos(df: pd.DataFrame, repository_filter: list[str]) -> pd.DataFrame:
    """Filter DataFrame by repository path, case insensitive."""
    if not repository_filter:
        return df
    pattern = "|".join(repository_filter)
    col = "repositoryPath" if "repositoryPath" in df.columns else df.columns[1]
    mask = df[col].str.contains(pattern, case=False, na=False)
    if "repositoryBranch" in df.columns:
        combined = df[col] + "/" + df["repositoryBranch"]
        mask = mask | combined.str.contains(pattern, case=False, na=False)
    return df[mask]


def empty_figure(message: str = "No data available for the selected filters.") -> go.Figure:
    """Create an empty figure with a centered message."""
    fig = go.Figure()
    fig.add_annotation(
        text=message,
        xref="paper",
        yref="paper",
        x=0.5,
        y=0.5,
        showarrow=False,
        font=dict(size=16, color="#666"),
    )
    fig.update_layout(
        xaxis=dict(visible=False),
        yaxis=dict(visible=False),
        height=400,
    )
    return fig


def short_repo(repo_path) -> str:
    """Extract short repo name from full path."""
    if not isinstance(repo_path, str):
        return str(repo_path)
    return repo_path.split("/")[-1] if "/" in repo_path else repo_path


def short_class(class_name) -> str:
    """Extract simple class name from FQN."""
    if not isinstance(class_name, str):
        return str(class_name)
    return class_name.split(".")[-1] if "." in class_name else class_name


def extract_package(class_name) -> str:
    """Extract package name from fully qualified class name."""
    if not isinstance(class_name, str):
        return ""
    parts = class_name.rsplit(".", 1)
    return parts[0] if len(parts) > 1 else ""


def compute_debt_score(df: pd.DataFrame) -> pd.Series:
    """Compute composite debt score from method quality metrics.

    Weighted combination of normalized complexity, cognitive complexity,
    nesting depth, and estimated bugs.
    """
    cols = {
        "cyclomaticComplexity": 0.4,
        "cognitiveComplexity": 0.3,
        "maxNestingDepth": 0.2,
        "halsteadEstimatedBugs": 0.1,
    }
    score = pd.Series(0.0, index=df.index)
    for col, weight in cols.items():
        if col in df.columns:
            col_max = df[col].max()
            if col_max > 0:
                score += weight * (df[col] / col_max)
    return score


def format_metric(name: str, value, thresholds: dict | None = None) -> str:
    """Format a metric value with executive-friendly label.

    Args:
        name: Metric name (e.g., 'wmc', 'lcom4', 'cbo')
        value: The metric value
        thresholds: Optional dict with 'healthy' threshold for comparison
    """
    labels = {
        "wmc": f"Complexity score: {value}",
        "lcom4": f"Should be split into {value} classes" if value > 1 else "Well-cohesive class",
        "tcc": f"Cohesion: {value:.0%}" if isinstance(value, float) else f"Cohesion: {value}",
        "cbo": f"Coupled to {value} other components",
        "maintainabilityIndex": f"Maintainability: {value:.0f}/100",
        "cyclomaticComplexity": f"Complexity: {value} paths",
        "cognitiveComplexity": f"Cognitive load: {value}",
        "maxNestingDepth": f"Nesting: {value} levels deep",
        "riskScore": f"Risk score: {value}",
        "instability": f"Instability: {value:.0%}",
        "abstractness": f"Abstractness: {value:.0%}",
        "distanceFromMainSequence": f"Architecture gap: {value:.2f}",
    }
    base = labels.get(name, f"{name}: {value}")

    if thresholds and name in thresholds:
        threshold = thresholds[name]
        if isinstance(value, (int, float)) and value > threshold:
            ratio = value / threshold
            base += f" ({ratio:.1f}x above healthy)"

    return base


# Default thresholds for "healthy" values
HEALTHY_THRESHOLDS = {
    "wmc": 10,
    "lcom4": 1,
    "cbo": 10,
    "cyclomaticComplexity": 10,
    "cognitiveComplexity": 15,
    "maxNestingDepth": 3,
    "riskScore": 10,
}


def add_repo_short(df: pd.DataFrame) -> pd.DataFrame:
    """Add a 'repoShort' column derived from repositoryPath."""
    if "repositoryPath" in df.columns:
        df = df.copy()
        df["repoShort"] = df["repositoryPath"].apply(short_repo)
    return df


def add_class_short(df: pd.DataFrame) -> pd.DataFrame:
    """Add 'classShort' and 'package' columns derived from className."""
    if "className" in df.columns:
        df = df.copy()
        df["classShort"] = df["className"].apply(short_class)
        df["package"] = df["className"].apply(extract_package)
    return df



from code_data_science import data_table as dt
from code_data_science.versions import index as index_versions
import plotly.graph_objects as go
import code_data_science.palette as palette

def create_violin_plot():
    df = dt.read_csv("../samples/dependency_usage_violin_nodejs.csv")
    df = df[["name", "requestedVersion"]]

    # make sure version is a string
    df["requestedVersion"] = df["requestedVersion"].astype(str)
    df["version"] = list(
        map(
            lambda v: v.removeprefix("^")
            .removeprefix("~")
            .removeprefix(">")
            .removeprefix("="),
            df.requestedVersion,
        )
    )

    vmap = index_versions(df.version)
    df["nVersion"] = list(map(lambda v: vmap[v], df.version))

    def index_name(names):
        sorted_names = sorted(list(set(names)))
        return {name: sorted_names.index(name) for name in sorted_names}

    nmap = index_name(df.name)
    df["nName"] = list(map(lambda g: nmap[g], df.name))

    df = df.sort_values(by=["nVersion", "nName"])

    colors = palette.colors_by_weight(500)

    fig = go.Figure()

    # Add a trace to the plot for each category
    for i, category in enumerate(df["nName"].unique()):
        category_data = df[df["nName"] == category]

        # Calculate counts for each dependency and version combination
        counts = (
            category_data.groupby("nVersion")["nName"].count().reset_index(name="count")
        )

        category_data_with_counts = category_data.merge(counts, on="nVersion")

        # Generate hover text including the count information
        hover_text = category_data_with_counts.apply(
            lambda row: f'<b>Package</b>: {row["name"]}<br><b>Version</b>: {row["version"]}<br><b>Count</b>: {row["count"]}',
            axis=1,
        )

        fig.add_trace(
            go.Scatter(
                x=category_data["nName"],
                y=category_data["nVersion"],
                mode="markers",
                marker=dict(color=colors[i % len(colors)], size=8),
                showlegend=False,
                name="",
                text=hover_text,
                hoverinfo="text",
                hoverlabel=dict(font=dict(size=18)),
            )
        )

        fig.add_trace(
            go.Violin(
                x=category_data["nName"],
                y=category_data["nVersion"],
                fillcolor="black",
                opacity=0.15,
                line_color="black",
                showlegend=False,
                width=0.7,
                bandwidth=0.4,
                hoverinfo="none",
                hoveron="points",
            )
        )

    num_versions = df["nVersion"].nunique()
    height_per_version = 32
    width_per_dependency = 80
    fig_height = max(num_versions * height_per_version, 900)
    fig_width = max(len(list(nmap.values())) * width_per_dependency, 900)
    tick_font_size = 13
    # Customizing the layout
    fig.update_layout(
        title="Package versions in use",
        xaxis_title="Packages",
        yaxis_title="Versions",
        height=fig_height,
        width=fig_width,
        xaxis=dict(
            tickfont=dict(size=tick_font_size),
            tickmode="array",
            tickvals=list(nmap.values()),
            ticktext=list(nmap.keys()),
        ),
        yaxis=dict(
            tickfont=dict(size=tick_font_size),
            tickmode="array",
            tickvals=list(vmap.values()),
            ticktext=list(vmap.keys()),
        ),
    )

    fig.show()
